@@ -117,22 +117,10 @@ def lfconvert(ui, src, dest, *pats, **opts):
 
 def _addchangeset(ui, rsrc, rdst, ctx, revmap):
  # Convert src parents to dst parents
-    parents = []
-    for p in ctx.parents():
-        parents.append(revmap[p.node()])
-    while len(parents) < 2:
-        parents.append(node.nullid)
+    parents = _convertparents(ctx, revmap)
 
     # Generate list of changed files
-    files = set(ctx.files())
-    if node.nullid not in parents:
-        mc = ctx.manifest()
-        mp1 = ctx.parents()[0].manifest()
-        mp2 = ctx.parents()[1].manifest()
-        files |= (set(mp1) | set(mp2)) - set(mc)
-        for f in mc:
-            if mc[f] != mp1.get(f, None) or mc[f] != mp2.get(f, None):
-                files.add(f)
+    files = _getchangedfiles(ctx, parents)
 
     def getfilectx(repo, memctx, f):
         if lfutil.standin(f) in files:
@@ -160,23 +148,7 @@ def _addchangeset(ui, rsrc, rdst, ctx, revmap):
             return context.memfilectx(f, data, 'l' in fctx.flags(),
                                       'x' in fctx.flags(), renamed)
         else:
-            try:
-                fctx = ctx.filectx(f)
-            except error.LookupError:
-                raise IOError()
-            renamed = fctx.renamed()
-            if renamed:
-                renamed = renamed[0]
-            data = fctx.data()
-            if f == '.hgtags':
-                newdata = []
-                for line in data.splitlines():
-                    id, name = line.split(' ', 1)
-                    newdata.append('%s %s\n' % (node.hex(revmap[node.bin(id)]),
-                        name))
-                data = ''.join(newdata)
-            return context.memfilectx(f, data, 'l' in fctx.flags(),
-                                      'x' in fctx.flags(), renamed)
+            return _getnormalcontext(repo.ui, ctx, f, revmap)
 
     dstfiles = []
     for file in files:
@@ -185,31 +157,15 @@ def _addchangeset(ui, rsrc, rdst, ctx, revmap):
         else:
             dstfiles.append(file)
     # Commit
-    mctx = context.memctx(rdst, parents, ctx.description(), dstfiles,
-                          getfilectx, ctx.user(), ctx.date(), ctx.extra())
-    ret = rdst.commitctx(mctx)
-    rdst.dirstate.setparents(ret)
-    revmap[ctx.node()] = rdst.changelog.tip()
+    _commitcontext(rdst, parents, ctx, dstfiles, getfilectx, revmap)
 
 def _lfconvert_addchangeset(rsrc, rdst, ctx, revmap, lfiles, normalfiles,
         matcher, size, lfiletohash):
     # Convert src parents to dst parents
-    parents = []
-    for p in ctx.parents():
-        parents.append(revmap[p.node()])
-    while len(parents) < 2:
-        parents.append(node.nullid)
+    parents = _convertparents(ctx, revmap)
 
     # Generate list of changed files
-    files = set(ctx.files())
-    if node.nullid not in parents:
-        mc = ctx.manifest()
-        mp1 = ctx.parents()[0].manifest()
-        mp2 = ctx.parents()[1].manifest()
-        files |= (set(mp1) | set(mp2)) - set(mc)
-        for f in mc:
-            if mc[f] != mp1.get(f, None) or mc[f] != mp2.get(f, None):
-                files.add(f)
+    files = _getchangedfiles(ctx, parents)
 
     dstfiles = []
     for f in files:
@@ -237,7 +193,9 @@ def _lfconvert_addchangeset(rsrc, rdst, ctx, revmap, lfiles, normalfiles,
             dstfiles.append(lfutil.standin(f))
             # largefile in manifest if it has not been removed/renamed
             if f in ctx.manifest():
-                if 'l' in ctx.filectx(f).flags():
+                fctx = ctx.filectx(f)
+                if 'l' in fctx.flags():
+                    renamed = fctx.renamed()
                     if renamed and renamed[0] in lfiles:
                         raise util.Abort(_('largefile %s becomes symlink') % f)
 
@@ -281,32 +239,80 @@ def _lfconvert_addchangeset(rsrc, rdst, ctx, revmap, lfiles, normalfiles,
             return context.memfilectx(f, lfiletohash[srcfname] + '\n', 'l' in
                 fctx.flags(), 'x' in fctx.flags(), renamed)
         else:
-            try:
-                fctx = ctx.filectx(f)
-            except error.LookupError:
-                raise IOError()
-            renamed = fctx.renamed()
-            if renamed:
-                renamed = renamed[0]
-
-            data = fctx.data()
-            if f == '.hgtags':
-                newdata = []
-                for line in data.splitlines():
-                    id, name = line.split(' ', 1)
-                    newdata.append('%s %s\n' % (node.hex(revmap[node.bin(id)]),
-                        name))
-                data = ''.join(newdata)
-            return context.memfilectx(f, data, 'l' in fctx.flags(),
-                                      'x' in fctx.flags(), renamed)
+            return _getnormalcontext(repo.ui, ctx, f, revmap)
 
     # Commit
+    _commitcontext(rdst, parents, ctx, dstfiles, getfilectx, revmap)
+
+def _commitcontext(rdst, parents, ctx, dstfiles, getfilectx, revmap):
     mctx = context.memctx(rdst, parents, ctx.description(), dstfiles,
                           getfilectx, ctx.user(), ctx.date(), ctx.extra())
     ret = rdst.commitctx(mctx)
     rdst.dirstate.setparents(ret)
     revmap[ctx.node()] = rdst.changelog.tip()
 
+# Generate list of changed files
+def _getchangedfiles(ctx, parents):
+    files = set(ctx.files())
+    if node.nullid not in parents:
+        mc = ctx.manifest()
+        mp1 = ctx.parents()[0].manifest()
+        mp2 = ctx.parents()[1].manifest()
+        files |= (set(mp1) | set(mp2)) - set(mc)
+        for f in mc:
+            if mc[f] != mp1.get(f, None) or mc[f] != mp2.get(f, None):
+                files.add(f)
+    return files
+
+# Convert src parents to dst parents
+def _convertparents(ctx, revmap):
+    parents = []
+    for p in ctx.parents():
+        parents.append(revmap[p.node()])
+    while len(parents) < 2:
+        parents.append(node.nullid)
+    return parents
+
+# Get memfilectx for a normal file
+def _getnormalcontext(ui, ctx, f, revmap):
+    try:
+        fctx = ctx.filectx(f)
+    except error.LookupError:
+        raise IOError()
+    renamed = fctx.renamed()
+    if renamed:
+        renamed = renamed[0]
+
+    data = fctx.data()
+    if f == '.hgtags':
+        data = _converttags (ui, revmap, data)
+    return context.memfilectx(f, data, 'l' in fctx.flags(),
+                              'x' in fctx.flags(), renamed)
+
+# Remap tag data using a revision map
+def _converttags(ui, revmap, data):
+    newdata = []
+    for line in data.splitlines():
+        try:
+            id, name = line.split(' ', 1)
+        except ValueError:
+            ui.warn(_('skipping incorrectly formatted tag %s\n'
+                % line))
+            continue
+        try:
+            newid = node.bin(id)
+        except TypeError:
+            ui.warn(_('skipping incorrectly formatted id %s\n'
+                % id))
+            continue
+        try:
+            newdata.append('%s %s\n' % (node.hex(revmap[newid]),
+                name))
+        except KeyError:
+            ui.warn(_('no mapping for id %s\n' % id))
+            continue
+    return ''.join(newdata)
+
 def _islfile(file, ctx, matcher, size):
     '''Return true if file should be considered a largefile, i.e.
     matcher matches it or it is larger than size.'''
@@ -369,7 +375,16 @@ def cachelfiles(ui, repo, node):
     toget = []
 
     for lfile in lfiles:
-        expectedhash = repo[node][lfutil.standin(lfile)].data().strip()
+        # If we are mid-merge, then we have to trust the standin that is in the
+        # working copy to have the correct hashvalue.  This is because the
+        # original hg.merge() already updated the standin as part of the normal
+        # merge process -- we just have to udpate the largefile to match.
+        if (getattr(repo, "_ismerging", False) and
+             os.path.exists(repo.wjoin(lfutil.standin(lfile)))):
+            expectedhash = lfutil.readstandin(repo, lfile)
+        else:
+            expectedhash = repo[node][lfutil.standin(lfile)].data().strip()
+
         # if it exists and its hash matches, it might have been locally
         # modified before updating and the user chose 'local'.  in this case,
         # it will not be in any store, so don't look for it.
@@ -451,11 +466,17 @@ def _updatelfile(repo, lfdirstate, lfile):
         # largefile is converted back to a normal file: the standin
         # disappears, but a new (normal) file appears as the lfile.
         if os.path.exists(abslfile) and lfile not in repo[None]:
-            os.unlink(abslfile)
+            util.unlinkpath(abslfile)
             ret = -1
     state = repo.dirstate[lfutil.standin(lfile)]
     if state == 'n':
-        lfdirstate.normal(lfile)
+        # When rebasing, we need to synchronize the standin and the largefile,
+        # because otherwise the largefile will get reverted.  But for commit's
+        # sake, we have to mark the file as unclean.
+        if getattr(repo, "_isrebasing", False):
+           lfdirstate.normallookup(lfile)
+        else:
+            lfdirstate.normal(lfile)
     elif state == 'r':
         lfdirstate.remove(lfile)
     elif state == 'a':
